//************************************************
//************ CPR related functions
//************************************************





//************************************************
//************************************************

// s2s_translation
// returns the input and output port lists from cpr and scsId
function [iPortList,oPortList] = C_getPortLists (cpr, id)
  nin = cpr.sim.inpptr(id+1) - cpr.sim.inpptr(id);  ///* number of input ports */
  nout = cpr.sim.outptr(id+1) - cpr.sim.outptr(id); ///* number of output ports */
  
  iPortList = list();
  for k=1:nin
    lprt = cpr.sim.inplnk(cpr.sim.inpptr(id)-1+k);
    inSz = cpr.sim.lnkptr(lprt+1) - cpr.sim.lnkptr(lprt);
    iPortList($+1) = inSz;
  end

  oPortList = list();
  for k=1:nout
    lprt = cpr.sim.outlnk(cpr.sim.outptr(id)-1+k);
    outSz = cpr.sim.lnkptr(lprt+1) - cpr.sim.lnkptr(lprt);
    oPortList($+1) = outSz;
  end
endfunction

// s2s_translation
function sdxType = C_getSdxBlkType (cpr,i)
if C_blkIsSensor (cpr, i) then 
  sdxType = "sensor";
elseif C_blkIsActuator (cpr, i) then 
  sdxType = "actuator";
elseif C_blkIsMemory (cpr, i) then
  // initial value has to be handled by scicos macros
  sdxType = "memory";
else
  sdxType = "algorithm";
end
endfunction

// ok
function funcName = C_getSensorFuncName ()
funcName = "sensor";
endfunction
function funcName = C_getActuatorFuncName ()
funcName = "actuator";
endfunction
function funcName = C_getClkRootFuncName ()
funcName = "clkRoot";
endfunction



// ok
function nb = C_getClkOutputPortNumber (cpr, i)
nb = (cpr.sim.clkptr(i+1)-cpr.sim.clkptr(i));
endfunction
function nb = C_getContinuousStatesNumber (cpr,i)
nb = cpr.sim.xptr(i+1)-cpr.sim.xptr(i);
endfunction



// s2s_translation
// get a coherent name for scicos block
// args:
// cpr, is scicos cpr
// i, is the scicos block id to process
function name = C_getCoherentName(cpr, i)
simu = cpr.sim;

if C_blkIsForbidden (cpr,i) then
  str = sprintf("block %d is forbidden",i) + "\n";
  // name = "scilab_Fct"; return; // remove // for DEBUG
  error(str);
end

// not type 4 functions may be present in the graph as long as user
// provide the corresponding type 4 C-code for it
if C_blkIsOfWrongType(cpr,i) then
  str = msprintf("block %s (id %d) not type 4, function not provided",..
      cpr.sim.funs(i), i);
  warning(str);
end

found = %F;
j=1;n=size(bllst);
oldname = simu.funs(i);
while j<=n & found==%F
  select oldname;
  case 'actionneur'+string(j) then
    name = C_getActuatorFuncName ();
    found = %T;
  case 'capteur'+string(j) then
    name = C_getSensorFuncName ();
    found = %T;
  case 'bidon' then
    name = C_getClkRootFuncName ();
    found = %T;
  end
  j = j+1;
end
if ~found then
  name = bllst(i).sim(1);
end
endfunction


// check if current block is handled by the translator,
// displays a warning if needed and return bool
function bool = C_blkIsForbidden(cpr,i)
bool = %F;
if C_blkIsZeroCrossing(cpr,i) then
  str = msprintf("Active zero crossing block is not supported (id:%d, name:%s)",i,cpr.sim.funs(i));
  warning(str);
  if C_getClkOutputPortNumber (cpr, i) <> 0 then
    bool = %T;
  end
elseif C_blkIsScilabFunction(cpr,i) then // 05_10_25 : scilab functions can t be processed
  warning("Block is a scilab function !");
  bool = %T;
// NOTE: continuous blocks now supported
//elseif C_blkIsContinuous(cpr,i) then
//  warning('Continuous state block is not supported');
//  bool = %T;
//elseif C_blkIsLogical(cpr,i) & cpr.sim.funtyp(i)>-1 & cpr.sim.funs(i)~='bidon' then
//  warning('Regular block generating activation not allowed yet');
//  bool = %T;
end
endfunction

function bool = C_blkIsLogical(cpr,i)
bool = C_getClkOutputPortNumber (cpr, i) <> 0; //(cpr.sim.clkptr(i+1)-cpr.sim.clkptr(i)) <>0;
endfunction
function bool = C_blkIsContinuous(cpr,i)
bool = C_getContinuousStatesNumber (cpr,i)<>0;
endfunction
function bool = C_blkIsZeroCrossing(cpr,i)
bool = (cpr.sim.zcptr(i+1)-cpr.sim.zcptr(i))<>0;
endfunction

function bool = C_blkIsOfWrongType(cpr,i)
bool = (cpr.sim.funtyp(i) <> 4);
endfunction
// 05_10_25 : added to detect scilab coded functions before testing function type
function bool = C_blkIsScilabFunction(cpr,i)
bool = (typeof(cpr.sim.funs(i)) == "function");
endfunction

// these two functions test on the number of input or output ports
// NOTE: it doesn t test the name and the block may not always be a real sensor
//       use C_blkIsTrueSensor(cpr,i) if you wan t to test the name
function bool = C_blkIsSensor(cpr,i)
nin = cpr.sim.inpptr(i+1) - cpr.sim.inpptr(i); ///* number of input ports */
bool = (nin == 0);
endfunction
function bool = C_blkIsActuator(cpr,i)
if C_blkIsLogical(cpr,i) then bool = %F; return; end
nout = cpr.sim.outptr(i+1) - cpr.sim.outptr(i); ///* number of output ports */
bool = (nout == 0);
endfunction

function bool = C_blkIsMemory(cpr,i)
bool = ~isempty(strindex(cpr.sim.funs(i),"dollar"));
endfunction


// these two functions test on the name of the function
function bool = C_blkIsTrueSensor(cpr,i)
res = grep(cpr.sim.funs(i),C_getSensorFuncName ());
bool = ~isempty(res);
endfunction
function bool = C_blkIsTrueActuator(cpr,i)
res = grep(cpr.sim.funs(i),C_getActuatorFuncName ());
bool = ~isempty(res);
endfunction

function bool = C_blkIsDiscrete(cpr,i)
bool = ((cpr.sim.zptr(i+1)-cpr.sim.zptr(i)) >1);
endfunction


function bool = C_blkIsClkRoot(cpr,i)
name = cpr.sim.funs(i);
if name == "clkRoot" | name == "bidon" then 
  bool = %T;
else
  bool = %F;
end
endfunction

