//********************************************************
//** this file contains functions 'linking' the different parts
//** of the translation process. For exemple, we must assert
//** we provide the same names to definitions from cpr estraction
//** to syndex file generation
//********************************************************





//********************************************************
//**** defName building functions ****//

//* name def building CORE (used by C_ and S2S_---_definition)
// standard block
function str = S2S_buildStdDefinitionName (funName, iPorts, oPorts)
str = funName + "_";
for i=1:length(iPorts)
str = str + msprintf("%d_",iPorts(i));
end
for i=1:length(oPorts)
str = str + msprintf("_%d",oPorts(i));
end
endfunction
// logical block: !! we need _scsId_ NOT defId !! 
// (defId is not always available, scsId is)
function str = S2S_buildLogDefinitionName (funName, scsId)
str = msprintf("%s_%d", funName, scsId);
endfunction


//********************************************************
//**** refName building functions ****//

// clkId needed for unicity in case of multiactivation in the same cond def
function name = cmn_getReferenceName (funName, scsId, clkId)
name = msprintf("%s_%d_%d",funName, scsId, clkId);
endfunction
function name = S2S_getIportName_port (ptId)
name = msprintf("in_%d",ptId);
endfunction
function name = S2S_getOportName_port (ptId)
name = msprintf("out_%d",ptId);
endfunction


//********************************************************
//** proxy ports naming
function surname = cmn_getProxyPortName (pBlkId, pPtId)
surname = msprintf("proxy_%d_%d",pBlkId, ptId);
endfunction
function name = cmn_getProxyIportName (pBlkId, pPtId)
name = "I_" + cmn_getProxyPortName (pBlkId, pPtId);
endfunction
function name = cmn_getProxyOportName (pBlkId, pPtId)
name = "O_" + cmn_getProxyPortName (pBlkId, pPtId);
endfunction


//********************************************************
//** generic functions

//** file function
function ok = cmn_writeToFile (fileName, str)
ok=%F;
[fd, err] = mopen(fileName, "w");
if err <> 0 then 
  str = sprintf("Error opening %s",fileName);
  warning (str);
  ok = %F;
  return;
end
mprintf("writing... ");
mfprintf(fd, str);
if (mclose(fd) ~= 0) then
  errStr = msprintf("could not write to file %s",fileName); 
  warning(errStr); 
end
mprintf("ok\n");
ok=%T;
endfunction


//** list functions

// I don't know if exists a O(1) function
function lst = C_arrayToList (array)
lst = list();
for i=1:size(array, 'r') 
  lst(i) = array(i,:);
end
endfunction

function ok = isInList (intList, val)
ok = %F;
for currVal=intList
  if currVal == val then ok = %T; return; end
end
endfunction

// O(n2)
function newList = purgeDoubles (oldList)
newList = list();
for elm=oldList
  found=%F; currId=1;
  while currId <= length(newList) & ~found 
    currElm = newList(currId);
    found = (elm == currElm);    
    currId = currId + 1;
  end
  if ~found then
    newList($+1) = elm;
  end
end
endfunction


function ok = cmn_isNumber (s)
ok = %T;
select typeof(s),
case "constant" then // should not happen
  warning("isNumber: constant variable should not happen");
  return; 
case "string" then 
  res = str2code(s);
  for i=1:length(res)
    n = res(i,1);
    if n<0 | (n>9 & n<>str2code('.')) then
      ok = %F; return;
    end
  end
else
  error("variable is not a number nor a string");
end
endfunction





function ok = cmn_isEmpty (s)
ok = %F;
if typeof(s) <> "string" then
  warning("isEmpty(s): variable is not of type string");
else
  ok = (s == emptystr());
end
endfunction