//***********************************************//
//******* create_m4x_file.sci
//***********************************************//



//**********************************//
//*** the code gen structure
//**********************************//
function codeGen = m4x_createVoid_codeGen ()
codeGen = tlist(["CodeGeneration";..
	"zArray";"nZ";.. // discrete states
	"rpptr";"rpar";"ipptr";"ipar";.. // real and integer parameters
	"x";..// continuous states
	"pasDiscret";"pasIntegration".. // discrete and continuous steps values
	],.. 
    list(), -1,..
    list(), list(), list(), list(),..
    list(),..
    0.1, 0.01);
endfunction

function codeGen = m4x_create_codeGen (cpr, appliZstep, appliSstep)
codeGen = m4x_createVoid_codeGen ();
codeGen = m4x_process_z_codeGen (codeGen, cpr);
codeGen = m4x_process_ipar_rpar_codeGen (codeGen, cpr);
codeGen = m4x_process_x_codeGen (codeGen, cpr);

codeGen.pasDiscret = appliZstep;
codeGen.pasIntegration = appliSstep;
endfunction

//**** x/xd : continuous states
function codeGen = m4x_process_x_codeGen (codeGen, cpr)
codeGen.x = cpr.state.x;
endfunction

//**** Zarray : contains the vector of discrete states needed by scicos blocks
function codeGen = m4x_process_z_codeGen (codeGen, cpr)
z=cpr.state.z; outtb=cpr.state.outtb; clkptr=cpr.sim.clkptr; iwa=zeros(clkptr($),1);
nZ = size(z,'*')+size(outtb,'*')+clkptr($);
nztotal=size(z,1);

Z=[z;outtb;iwa];

codeGen.zArray = Z;
codeGen.nZ = nZ;
endfunction

//**** rpar/ipar : inner parameters for scicos blocks
function codeGen = m4x_process_ipar_rpar_codeGen (codeGen, cpr)
codeGen.rpptr = cpr.sim.rpptr;
codeGen.ipptr = cpr.sim.ipptr;
codeGen.rpar = cpr.sim.rpar;
codeGen.ipar = cpr.sim.ipar;
endfunction
//**************************************//

//** this file contains functions to build the appli.m4x file contents
//** It uses the codeGen and s2sMain struct

// MODIF 22/12/05
// function nb = m4x_getNumberOfIterations ()
// nb = 5;
// endfunction
//function path = m4x_getScicosFuncPath ()
//path = "SCICOS_FILES";
//endfunction

// file which should include all code to be included at the top of every .c files
function fileName = m4x_getCommonFctFileName ()
fileName = "s2s_common.c";
endfunction

//**** MAIN FUNCTION ****//
//** args:
// s2sMain, is the completed S2S_main struct
// cpr, is the result of cpass2
// str, is the string that will be copied in the appli.m4x file

function str = create_m4x_file (s2sMain, cpr, appliIteNumber, appliZstep, appliSstep, appliScsFuncPath)
nBlk = S2S_getBlockListLength_main (s2sMain);
// codeGen, is the completed CodeGeneration struct
codeGen = m4x_create_codeGen (cpr, appliZstep, appliSstep); // nBlk);


str =..
    "dnl *******************************\n"+..
    "dnl BEGINNING OF M4X GENERATED FILE\n"+..
    "dnl *******************************\n"+..
    m4x_getHeader (appliIteNumber) +..
    m4x_getProcInitMacro (codeGen, nBlk) +..
    m4x_getDefinitionMacros (s2sMain) +..
    m4x_getFooter ();
endfunction


/// ************************************** ///
/// header / footer of m4x file
function str = m4x_getHeader (appliIteNumber)
// nbIter = m4x_getNumberOfIterations (); // MODIF 22/12/05
str = "divert(-1)\n" + "define(`NOTRACEDEF'')\n" +.. 
    msprintf("define(`NBITERATIONS'',``%d'''')",appliIteNumber) +..
    "\ndefine(`dnldnl'',``// '''')\n\n" +..
    "\ndefine(`    # '',``// '''')\n\n";
endfunction



/// ************************************** ///
/// functions related to proc init macro
function str = m4x_getProcInitMacro (codeGen, nBlk)

str = "define(`proc_init_'',``\n"+..
    "/*********************************************\n"+..
    " * proc init : main block struct declaration\n"+..
    " *********************************************/\n\n" +..
    msprintf("scicos_block blocks[%d];", nBlk) + "\n" +..
    m4x_getZarrayStr (codeGen) + "\n" +..
    m4x_get_ipar_rparStr (codeGen) + "\n" +..
    m4x_getContinuousDeclStr (codeGen) + "\n" +..
    "\n '''')\n\n";


// init of blk[i] is no longer made here (made in macro init)
endfunction





/// ************************************** ///
/// definition processing ///

// main function gathering all macro definitions
function str = m4x_getDefinitionMacros (s2sMain)
nbDef = S2S_getLastDefinitionId_main (s2sMain);


// including global macros : ifthel, eselect and remanence
// TODO: more genericity!
str = "\n"+..
    "/* generic macros useful in case of hierarchical activation\n"+..
    " * WARNING: in case of home made logical blocks, user must\n"+..
    " * provide macro for associated test block */\n"+..
    m4x_getLogBlkDefMacroStr () +..
    m4x_getRemanenceMacroStr () +..
    "\n\n/* macro definition of each function used */\n\n";

initStr = m4x_getBlockStructInitStr ();

mprintf("**** BUILDING DEF MACROS (tot:%d) :\n****defId:", nbDef);



// this part fills the 'blocks' structure for each block
for defId=1:nbDef
  mprintf("%d ", defId);
  if ~S2S_isLogical_definition (s2sMain, defId) then     
    str = str + m4x_getBlockMacroStr (initStr, s2sMain, defId) + "\n";
  end
end // end for
mprintf("\n");
endfunction

// build macro def for defId
function str = m4x_getBlockMacroStr (structInitStr, s2sMain, defId)
defName = S2S_getName_definition (s2sMain, defId);
funName = S2S_getFunName_definition (s2sMain, defId);

nin  = length(S2S_getIportList_definition (s2sMain, defId));
nout = length(S2S_getOportList_definition (s2sMain, defId));

blkStr = msprintf("blocks[$1-1]");

initStr = "/* init phase for block " + defName + "  */\n"+..
    structInitStr +..
    m4x_getPortLinkingStr (%T, nin, nout) +..
    msprintf("%s(&%s, flag_init);",funName,blkStr)+ "\n"+..
    msprintf("%s(&%s, flag_reinit);",funName,blkStr)+ "\n";

loopStr = "/* loop phase for block " + defName + "  */\n"+..
    m4x_getPortLinkingStr (%F, nin, nout) +..
    msprintf("%s(&%s, flag_updateOutputs);",funName,blkStr)+ "\n"+..
    msprintf("%s(&%s, flag_updateZstates);",funName,blkStr)+ "\n"+..
    m4x_getContinuousLoopStr (s2sMain, defId);
// TODO: insert here the continu treatment if a continuous state exists for def defId
// => need modification of parameters in def/ref

endStr = "/* end phase for block " + defName + "  */\n"+..
    m4x_getPortLinkingStr (%F, nin, nout) +..
    msprintf("%s(&%s, flag_end);",funName,blkStr)+ "\n";

str = m4x_getMacroStr (defName, initStr, loopStr, endStr);
endfunction

// build the code corresponding to the integration in the loop of a continuous block
function str = m4x_getContinuousLoopStr (s2sMain, defId)
// init
str = emptystr();
// test if defId is continuous
if S2S_isContinuous_definition (s2sMain, defId) then
  funName = S2S_getFunName_definition (s2sMain, defId);
  // if the continuous code is needed, create it
  str = str +..
      "/* block had a continuous state */\n"+..
      "{\n  int iter;\n"+..
      "  ti = i * pasDiscret;\n  tout = ti;\n"+..
      "  while (tout < ti + pasDiscret) {\n"+..
      "    /* computing xdot */ \n"+..
      "    " + funName + "(&blocks[$1-1],flag_updateSstates);\n"+..
      "    /* S state is a vector, integrate each element */\n"+..
      "    neq = blocks[$1-1].nx;\n"+..
      "    /* integrate via Euler method */\n"+..
      "    for (iter=0;iter<neq;iter++)\n"+..
      "      blocks[$1-1].x[iter] = blocks[$1-1].x[iter] + "+..
      "pasIntegration * blocks[$1-1].xd[iter];\n"+..
      "    tout += pasIntegration;\n"+..
      "  } /* end while */\n"+..
      "  /* proceed to one final integration step */\n"+..
      "  tout = tout - pasIntegration;\n"+..
      "  pasFinalIntegration = ti + pasDiscret - tout + pasIntegration;\n"+..
      "  for (iter=0;iter<neq;iter++)\n"+..
      "      blocks[$1-1].x[iter] = blocks[$1-1].x[iter] + "+..
      "pasFinalIntegration * blocks[$1-1].xd[iter];\n"+..
      "} /* end of integration for block $1 */\n";
end
endfunction


// initPhase, is a bool
function str = m4x_getPortLinkingStr (initPhase, nin, nout)
str = emptystr();
IportOffset = param_getOffsetOfPortPosition();
OportOffset = IportOffset + nin;

for k=1:nin
  portId = IportOffset + k;
  // linking scicos structure and syndex core
  str = str +..
      msprintf("%s.inptr[%d] = $%d;",blkStr, k-1, portId)+ "\n";
  if initPhase then
    str = str +..
	msprintf("%s.insz[%d] = ''$%d_size_`;",blkStr, k-1, portId) + "\n";
  end
  // TODO : get each port initialization via outtb..
  // should init the sdx pointer _value_ to the initial outtb _value_
end

// O port init
for k=1:nout
  portId = OportOffset + k;
  str = str +..
      msprintf("%s.outptr[%d] = $%d;", blkStr, k-1, portId)+ "\n";
  if initPhase then
    str = str +..
	msprintf("%s.outsz[%d] = ''$%d_size_`;", blkStr, k-1, portId)+ "\n";
  end
end
endfunction

function str = m4x_getMacroStr (macroName, initStr, loopStr, endStr)
str =..
    "define(`" + macroName + "'',`ifelse(\n" +..
    "MGC,`MGC'',``'''',\n" +..
    "MGC,`INIT'',``" + initStr + "'''',\n" +..
    "MGC,`LOOP'',``" + loopStr + "'''',\n" +..
    "MGC,`END'',``" + endStr + "'''')'')\n";
endfunction

// NOTE: add here others log block def if needed
// TODO: should be more generic
function str = m4x_getLogBlkDefMacroStr ()
codeStr = "$2[0] = ($1[0]>0 ? 1 : 2);";
str = m4x_getMacroStr (sdx_buildLogBlkDefName ("ifthel"), "", codeStr , "")+..
    m4x_getMacroStr (sdx_buildLogBlkDefName ("eselect"), "", codeStr , "");
endfunction

function str = m4x_getRemanenceMacroStr ()
codeStr =..
    "{" +..
    "int i;" +..
    "for (i=0;i<$1;i++)" +..
    "  $3[i] = $2[i];" +..
    "}";
str = m4x_getMacroStr (sdx_getRemanenceDefName (), "", codeStr , "");
endfunction



function str = m4x_getFooter (s2sMain, appliScsFuncPath)
// this part fills what will be included at the very beginning of the C generated file
str = "\n\ndnl *********************\ndnl end of file\ndnl *********************" +..
    "\n\ndivert\n#include <stdio.h> /* for printf */\n"+..
    "`enum flag_type { flag_init = 4, flag_updateOutputs = 1, flag_updateZstates = 2, flag_updateSstates = 0, flag_reinit = 6, flag_end = 5};''\n\n"+..
    "/******* COMMON ERRORS **********\n"+..
    "* 1/ if you forgot to add the init phase to the proc generation phases, the `proc_init_'' macro have not\n"+..
    "*    been generated. Manually add its contents here\n"+..
    "* 2/ lines begginning by the # character may exists if you are using the beta version of\n"+..
    "*    SynDEx code generation. Delete them.\n"+..
    "********************************/\n\n"+..
    "\ndnl DONT FORGET TO CHECK IF ALL THE FILES BELOW ARE AVAILABLE\n" +..
    "\ndnl Including common C file\n" +..
    msprintf("include(%s/%s)", appliScsFuncPath, m4x_getCommonFctFileName()) + "\n";

// including function bodies via m4
// the included files are the files corresponding to the scicos blocks
funList = m4x_getFunListToInclude (s2sMain);
for i=1:length(funList)
  funName = funList(i);
  str = str + msprintf("include(%s/%s.c)",appliScsFuncPath,funName)+"\n";
end
// always include the common c file
str = str + "\ndivert(-1)\ndivert`''dnl ---------------- end of file ------------------\n\n";
endfunction


function funList = m4x_getFunListToInclude (s2sMain)
nbDef = S2S_getLastDefinitionId_main (s2sMain);
funList = list();
for defId=1:nbDef
  if ~m4x_isUserDefined_definition (s2sMain, defId) then
    funList($+1) = S2S_getFunName_definition (s2sMain, defId);
  end
end
funList = purgeDoubles (funList);
endfunction

function ok = m4x_isUserDefined_definition (s2sMain, defId)
funName = S2S_getFunName_definition (s2sMain, defId);

ok = S2S_isLogical_definition (s2sMain, defId) |..
    funName == C_getSensorFuncName () |..
    funName == C_getActuatorFuncName();
endfunction


//*********************************//

function str = m4x_getZarrayStr (codeGen)
Z = codeGen.zArray;
nZ = codeGen.nZ;

// useful for discrete blocks. global allowed because
// of non multiactivation of discrete blocks
// For information, z=[z;outtb;iwa]. only z is used for now
str = "double z[] = {" + strcat(string(Z),",")+"};";
// not used by SynDEx
// void **work;
// work = z + msprintf('%d',nZ)
endfunction


function str = m4x_getContinuousDeclStr (codeGen)
x = codeGen.x;
if isempty(x) then
  str = "/* no continuous state, creating void vectors */\ndouble xt[]={0};\ndouble xtd[]={0};";
else  
  str =..
      'int neq;\n'+..
      'double ti, tout, pasFinalIntegration;\n'+..
      msprintf("double pasDiscret = %f;",codeGen.pasDiscret)+ "\n" +..
      msprintf("double pasIntegration = %f;",codeGen.pasIntegration)+ "\n" +..
      'double xt[]={'+strcat(string(x),',')+'};\n' +..
      'double xtd[]={'+strcat(string(x),',')+'};';
end
endfunction

// inspired by Rachid Djenidi 's work about code generation
function str = m4x_get_ipar_rparStr (codeGen)
str = emptystr();

rpptr = codeGen.rpptr;
ipptr = codeGen.ipptr;
rpar = codeGen.rpar;
ipar = codeGen.ipar;

// rpar processing
if size(rpar,1) <> 0 then
  str = str + 'static double RPAR1[ ] = {';

  mprintf("some blocks have RPAR:");
  for i=1:(length(rpptr)-1)
    if rpptr(i+1) - rpptr(i)>0 then
      mprintf("%d ",i);
      str = str + strcat(string(rpar(rpptr(i):rpptr(i+1)-1))+',');
    end
  end
  mprintf("\n");
  str = str + '};\n';
else
  str = str + 'static double RPAR1[1];\n';
end

// ipar processing
if size(ipar,1) <> 0 then
  str = str + 'static int IPAR1[ ] = {';

  mprintf("some blocks have IPAR:");
  for i=1:(length(ipptr)-1) 
    if ipptr(i+1) - ipptr(i)>0  then
      mprintf("%d ",i);
      str = str + strcat(string(ipar(ipptr(i):ipptr(i+1)-1))+',');
    end
  end
  mprintf("\n");
  str = str + '};\n';
else
  str = str + 'static integer IPAR1[1];\n';
end
endfunction


//***************************************//

// generate the string of all the initialisations that can be made from the block
// Since we only need the param _names_, this part is constant for all blocks
function str = m4x_getBlockStructInitStr ()
str = emptystr();

blkStr = msprintf("blocks[$1-1]");

offset = param_getOffsetOfParamPosition(); // first parameter is the scsId

paramNameList = param_getDefinitionParameterList ();
strBeforeList = param_getStrBeforeList ();
strAfterList = param_getStrAfterList ();


for fieldId=1:(length(paramNameList)- offset)// length paramNameList+1=strBeforeList=strAfterList
  fieldName = paramNameList(fieldId + offset);  // + offset because of scsId
  befStr = strBeforeList(fieldId);
  aftStr = strAfterList (fieldId);
  
  fieldValue = msprintf("$%d", offset + fieldId);
  str = str +..
      msprintf("%s.%s = %s%s%s;",blkStr,fieldName,befStr,fieldValue,aftStr) + "\n";
end

str = str +..
    msprintf("%s.nevprt = 0;",blkStr)+ "\n" +..
    msprintf("if ((%s.evout = calloc(%s.nevout,sizeof(double)))== NULL ) return 0;",blkStr,blkStr) + "\n"+..
    msprintf("if ((%s.insz = malloc(sizeof(int) * %s.nin))== NULL ) return 0;",blkStr,blkStr) + "\n"+..
    msprintf("if ((%s.inptr = malloc(sizeof(double*) * %s.nin))== NULL ) return 0;",blkStr,blkStr) + "\n"+..
    msprintf("if ((%s.outsz = malloc(sizeof(int) * %s.nout))== NULL ) return 0;",blkStr,blkStr) + "\n"+..
    msprintf("if ((%s.outptr = malloc(sizeof(double*) * %s.nout))== NULL ) return 0;",blkStr,blkStr) + "\n"+..
    msprintf("%s.work=NULL;",blkStr)+ "\n";
endfunction

/// ********** END OF FILE ******************** ///