function dbg = main_getDebugMode ()
dbg = %F;
endfunction

function ok = main_translator (cpr, clkMat, dataMat)
ok = %F;

mprintf("\n" +..
    "*********************************************\n"+..
    " preprocessing the scicos structure resulting of precompilation\n"+..
    "*********************************************\n\n");
// preprocessing of data from scicos cpass2
[newCpr, clkMat, dataMat, clkRootId] = scicosPreprocess(cpr, clkMat, dataMat);

mprintf("\n" +..
    "*********************************************\n"+..
    " processing control flow into data flow \n"+..
    "*********************************************\n\n");

// graph translation
s2sMain = s2s_translation (newCpr, clkMat, dataMat, clkRootId);

mprintf("\n" +..
    "*********************************************\n"+..
    " converting s2s structure into sdx structure \n"+..
    "*********************************************\n\n");



// building the sdx struct from s2sMain struct
sdxApp = s2sMain_to_sdxApp (s2sMain);

// getting sdx file
sdxStr = sdx_mainApp_toString (sdxApp);

mprintf("\n" +..
    "*********************************************\n"+..
    " building associated code generation files \n"+..
    "*********************************************\n\n");


// ************* ASKING PATHS ******************* //
if (main_getDebugMode ()) then
  appliPath = "/afs/inria.fr/rocq/home/aoste/cfaure/TMP_SYNDEX";
  appliName = "my_app";
  sdxPath = "/local/cfaure/syndex-6.8.0/macros";
  appliScsFuncPath = "SCICOS_FILES";
  appliIteNumber = 10;
  appliZstep = 0.01;
  appliSstep = 0.0001;
else
  ask_ok = %F;
  while ~ask_ok do
    app = main_askPaths ();
    if isempty(app) then // cancel button hit -> return
      warning("Cancelling file creation");
      ok = %F;
      return;
    end
    ask_ok = main_checkPaths (app);
  end
  
  // process user data
  appliPath = app(1); appliName = app(2); sdxPath = app(3); appliScsFuncPath = app(7);
 
  appliIteNumber = msscanf(app(4),"%d");
  appliZstep = msscanf(app(5),"%f");
  appliSstep = msscanf(app(6),"%f");
end

// build sdx file name
if part(appliPath,length(appliPath)) <> "/" then 
  sdxFileName = appliPath + "/" + appliName + ".sdx";
else
  sdxFileName = appliPath + appliName + ".sdx";
end

// getting appli.m4x
m4xStr = create_m4x_file (s2sMain, newCpr, appliIteNumber, appliZstep, appliSstep, appliScsFuncPath);


// ************* WRITING FILES ****************** //
mprintf("**** writing SDX file : \n");
cmn_writeToFile (sdxFileName, sdxStr);
sdxStr = emptystr();

mprintf("**** writing M4X file : \n");
cmn_writeToFile (appliPath + "/" + appliName + ".m4x", m4xStr);
m4xStr = emptystr();

mprintf("**** writing other m4 files : \n");
if ~create_codeGen_files (appliPath, sdxPath, appliName) then
  warning("problem occured");
end

mprintf("ALL DONE.\n\n");
ok = %T;
endfunction




// *********************************************
// path resolution and file writing functions
// *********************************************
function data = main_askPaths ()
data = x_mdialog("Enter :",[..
	"Absolute application path";..
	"Application name (without suffix)";..
	"SynDEx macro path";..
	"Number of iterations";"Discrete step value";"Continuous step value";..
	"Scicos C functions path"..
	],[..
	"~/";..
	"my_application";..
	"$(HOME)/syndex/macros";..
	"2000";"0.01";"0.0005";..
	"SCICOS_FILES"..
	]);
endfunction

function ok = main_checkPaths (app)
ok = %T;
n=size(app,'r');
for i=1:n
  appField = app(i);
  if cmn_isEmpty (appField);
    str = msprintf ("field %d empty",i);
    warning(str); ok = %F; return;
  end
end

// check if numeric fields are not strings
if ~cmn_isNumber(app(4)) | ~cmn_isNumber(app(5)) | ~cmn_isNumber(app(6)) then
  str = msprintf ("requested numeric field is not a number");
  warning(str); ok = %F; return;
end
endfunction