//** this file contains all functions needed to retrieve block parameters
// these parameters must be given to the sdx reference and used in the appli.m4x


//*********************
//**** offset constants

// in appli.m4x, the parameter structure is :
// blkId, 
// param1 param2..m4x_getOffsetOfPortPosition(), 
// Iport1, Iport2.. IportN,
// Oport1, Oport2.. OportM

// returning the position of the first input port in the
// definition macro call i.e, number of def parameters
function offset = param_getOffsetOfPortPosition()
// param_getDefinitionParameterList already includes scsId
offset = size(param_getDefinitionParameterList ()); //  + param_getScsIdRow ();
endfunction

function offset = param_getOffsetOfParamPosition()
offset = param_getScsIdRow ();
endfunction

function paramId = param_getScsIdRow ()
paramId = 1;
endfunction

//used in appli.m4x
//**** functions dedicated to the scs blk struct init
function paramNameList = param_getDefinitionParameterList ()
paramNameList = list( "scsId",.. // usefull for ref init
    "type","ztyp",..
    "ng","nz",..
    "nrpar","nipar",..
    "nevout","nmode",..
    "z","rpar","ipar",..
    "x","xd","nx",..
    "nin", "nout");
endfunction

// used to fill sdx reference parameters NOT in appli.m4x
// WARNING: fields are computed for any block type 
//          (non continuous blocks gets x and xd pointer)
function fieldValueList = param_getParamValueList (newCpr, scsId)
fieldValueList = list(..
    scsId,..
    newCpr.sim.funtyp(scsId),.. // type
    newCpr.sim.ztyp(scsId),.. // ztyp
    newCpr.sim.zcptr(scsId+1)-newCpr.sim.zcptr(scsId),.. // ng
    newCpr.sim.zptr(scsId+1)-newCpr.sim.zptr(scsId),.. // nz
    newCpr.sim.rpptr(scsId+1)-newCpr.sim.rpptr(scsId),.. // nrpar
    newCpr.sim.ipptr(scsId+1)-newCpr.sim.ipptr(scsId),.. // nipar
    newCpr.sim.clkptr(scsId+1)-newCpr.sim.clkptr(scsId),.. // nevout
    newCpr.sim.modptr(scsId+1)-newCpr.sim.modptr(scsId),.. // nmode
    newCpr.sim.zptr(scsId)-1,.. // z (in fact pointer to z)
    newCpr.sim.rpptr(scsId)-1,.. // rpar (pointer to rpar)
    newCpr.sim.ipptr(scsId)-1,.. // ipar (pointer to ipar)
    newCpr.sim.xptr(scsId)-1,.. // x for continuous states
    newCpr.sim.xptr(scsId)-1,.. // xd for continuous states
    newCpr.sim.xptr(scsId+1) - newCpr.sim.xptr(scsId),.. // nx for continuous states
    newCpr.sim.inpptr(scsId+1) - newCpr.sim.inpptr(scsId),.. // nin
    newCpr.sim.outptr(scsId+1) - newCpr.sim.outptr(scsId).. // nout
    ); 
endfunction


//NOTE the length of strBeforeList is different from the paramNameList
//   because the scsId is given first
function strBeforeList = param_getStrBeforeList ()
strBeforeList = list(..
    "","",..
    "","",..
    "","",..
    "","","&(z[",..
    "&RPAR1[","&IPAR1[",..
    "&(xt[","&(xtd[","",.. // x, xd, nx
    "","");
endfunction
function strAfterList = param_getStrAfterList ()
strAfterList = list(..
    "","",..
    "","",..
    "","",..
    "","","])",..
    "]","]",..
    "])","])","",..
    "","");
endfunction
