// this file defines structures matching the syndex file grammar



/////////////////////////// port ////////////////////////////:
// args:

// type: type of data user defined(string)
// size: size of data (constant integer)
// name: port name (string)

// [posX, posY]: constant integer, constant integer

// creation functions
function sdxPort = sdx_createVoid_port ()
sdxPort = sdx_create_port (emptystr(), -1, emptystr());
endfunction

// direction and order no more needed, computed at toString
function sdxPort = sdx_create_port (ptType, ptSize, name)
sdxPort = tlist (["Sdx_port";"dataType";"dataSize";"name"],..
    ptType, ptSize, name);
endfunction


// add functions
function pt = sdx_setDataType_port (pt, pttype)
pt.dataType = pttype;
endfunction
// if string, then port is parametered
function pt = sdx_setDataSize_port (pt, ptsize)
pt.dataSize = ptsize; // msprintf("%d",ptsize);
endfunction
function pt = sdx_setName_port (pt, name)
pt.name = name;
endfunction


// one must give the 'order' field
function str = sdx_port_toString (port, order)
// size may be parametrized or not. I.e, the type of port.size may be a string or a constant
ptSize = var_toString(port.dataSize);

str = msprintf("%s[%s] %s %d;",port.dataType,ptSize,port.name,order);
endfunction

//////////////////////////////////////////////////////////////////


//////////////////////////// reference //////////////////////////
// args:
// definition : we only store defName or defId. but user may also provide a 
// *completed* definition of the ref because of data redundancy (sdx_definition).
// Some coherency checking is then provided.
// name: name of the ref (string)
// parameters: a list of sdx_paramter 
// posX, posY: constant integer (not used for now)
//

function sdxRef = sdx_create_reference (definition, name, parameters)
sdxRef = sdx_createVoid_reference ();
sdxRef = sdx_setName_reference (sdxRef, name);
sdxRef = sdx_addParameters_reference (sdxRef, parameters);
sdxRef = sdx_setDef_reference (sdxRef,definition);
endfunction

function sdxRef = sdx_createVoid_reference ()
sdxRef = tlist (["Sdx_reference";"definition";"name";"parametersValue";"posX";"posY"],..
    "", "", list(), "", "");
endfunction


// set/add functions

// definition can be either a string(name of the def)/a constant (def id)/def object
function ref = sdx_setDef_reference (ref,definition)
ref.definition = definition; 
endfunction

function ref = sdx_setName_reference (ref, name)
ref.name = name;
endfunction

// paramList is a list of parameters
// a parameter may be: a string/constant/list of string/constant
function ref = sdx_addParameters_reference (ref, paramList)
for param=paramList
  ref.parametersValue($+1) = param; // ref = sdx_addParameter_reference (ref, param);
end
endfunction


// to string
function str = sdx_reference_toString (ref)
if typeof(ref.definition) == "string" then
  defStr = ref.definition;
elseif sdx_isA_definition (ref.definition) then // assume is a def object
  if ref.definition.library <> emptystr() then 
    defStr = msprintf("%s/%s",ref.definition.library, ref.definition.name); 
  else
    defStr = ref.definition.name;
  end
end

paramStr = sdx_parameters_toString (ref.parametersValue);

str = defStr + paramStr + " " + ref.name + ";";
endfunction
/////////////////////////////////////////////////////::


////////////// definitions /////////////////////
// TODO : explanation about genericity
function sdxDef = sdx_create_definition ()
error("not a proper function, use specific creation functions");
endfunction

// generic definition creation (private)
function sdxDef = sdx_createVoid_definition (defType)
sdxDef = tlist ([defType;..
	"algoType";"library";"name";"parameters";"IportList";"OportList";..
	"refList";"depList";..
	"condSubgraphList";"condPortId";..
	"sizeX";"sizeY"],..
    emptystr(), emptystr(), emptystr(), list(), list(), list(),.. // core
    list(), list(),.. // subgraph related fields
    list(), -1,.. // conditionned subgraph related fields
    emptystr(), emptystr()); // useless for now
endfunction

// specific definition creation (public)
function sdxDef = sdx_createVoid_atomicDefinition ()
sdxDef = sdx_createVoid_definition ("Sdx_atomic_definition");
endfunction

function sdxDef = sdx_createVoid_subgraphDefinition ()
sdxDef = sdx_createVoid_definition ("Sdx_subgraph_definition");
sdxDef = sdx_setAlgoType_definition (sdxDef, "algorithm");
endfunction

function sdxDef = sdx_createVoid_conditionnedDefinition ()
sdxDef = sdx_createVoid_definition ("Sdx_conditionned_definition");
sdxDef = sdx_setAlgoType_definition (sdxDef, "algorithm");
sdxDef = sdx_fastAddCondPort_definition (sdxDef);
endfunction

function def = sdx_fastAddCondPort_definition (def)
if typeof(def) <> "Sdx_conditionned_definition" then
  error ("cannot add a cond port on a non conditionned definition");
end
// TODO: generalize the port names
condPort = sdx_create_port ("int", 1, "in_1"); // adding a default cond port, always the first
def = sdx_addIport_definition (def, condPort);
condPortId = length(def.IportList);
def = sdx_setCondPortId_definition (def, condPortId);
endfunction

function sdxDef = sdx_create_libraryDefinition (sdxLib)
sdxDef = sdx_createVoid_definition ("Sdx_library_definition");
sdxDef = sdx_setLibrary_definition (sdxDef, sdxLib);
endfunction

//*** general test functions ***//
function ok = sdx_isA_definition (def)
ok = %T;
defType = typeof(def);
if  defType<>"Sdx_conditionned_definition" &..
    defType<>"Sdx_subgraph_definition" &..
    defType<>"Sdx_atomic_definition" &..
    defType<>"Sdx_library_definition" then
ok = %F;
end
endfunction

function ok = sdx_isAtomic_definition (def)
ok = typeof(def) == "Sdx_atomic_definition";
endfunction

function ok = sdx_isA_libraryDefinition (def)
if typeof(def)<>"Sdx_library_definition" then ok = %T; else ok = %F; end
endfunction

function ok = sdx_existsPort_definition (def, portName)
ok = %F;
if def.condPort.name == portName then
  warning("sdx_existsPort_definition: port is the cond port");
  ok = %T;
  return;
end
for pt=def.IportList
  if pt.name == portName then
    ok = %T;
    return;
  end
end
for pt=def.OportList
  if pt.name == portName then
    ok = %T;
    return;
  end
end
endfunction

//*** general set/add functions ***//
function def = sdx_setAlgoType_definition (def, algotype)
if algotype<>"algorithm" & algotype<>"sensor".. 
    & algotype<>"actuator" & algotype<>"memory" then
  error("illegal definition algotype");
else
  def.algoType = algotype;
end
endfunction

function def = sdx_setName_definition (def,name)
if typeof(name) <> "string" then error("incorrect type for name"); end 
def.name = name;
endfunction

// params MUST be strings
// this function is pretty complicated in order to cover most kind of cases
function def = sdx_addParameter_definition (def, varargin)
select length(varargin),
case 0 then
  error ("trying to add a void parameter");
case 1 then // parameter may be a list
  param = varargin(1);
  for p=param
    if isNumber(p) then
      errStr = msprintf("cannot add a scalar parameter to a definition");
      error (errStr);
    end
    def.parameters($+1) = p;
  end
else
  for i=1:size(varargin)
    param = varargin(i);
    if isNumber(param) then
      errStr = msprintf("cannot add a scalar parameter to a definition");
      error (errStr);
    end
    def.parameters($+1) = param;
  end
end
endfunction

function def = sdx_addParameters_definition (def, paramList)
for param=paramList
  def = sdx_addParameter_definition (def, param);
end
endfunction

function def = sdx_addIport_definition (def, port)
// for parameter sized port, we should check if param exists in def
def.IportList($+1) = port;
endfunction
function def = sdx_addOport_definition (def, port)
// for parameter sized port, we should check if param exists in def
def.OportList($+1) = port;
endfunction

// specific to scicos 2 syndex traduction
function def = sdx_fastAddIport_definition (def, ptsize)
ptName = msprintf("in_%d",length(def.IportList)+1);
pt = sdx_create_port ("double", ptsize, ptName);
def = sdx_addIport_definition (def, pt);
endfunction
function def = sdx_fastAddOport_definition (def, ptsize)
ptName = msprintf("out_%d",length(def.OportList)+1);
pt = sdx_create_port ("double", ptsize, ptName);
def = sdx_addOport_definition (def, pt);
endfunction

// add/set functions to facilitate non conditionned subgraphs manipulation
// in order to retrieve the Id of either dep or rep, one should use :
// id = length(def.[dep|ref]List);
//
// the varargin optionnal parameter is used to specify a condSG id if needed
// Assuming csg exists
function def = sdx_addRef_definition (def, ref, varargin)
def.refList($+1) = ref;

if length(varargin) == 1 then
  csgId = varargin(1); refId = length(def.refList);
  csg = def.condSubgraphList(csgId);
  csg = sdx_addRef_condSubgraph (csg, refId);
  def.condSubgraphList(csgId) = csg;
end
endfunction

// varargin may be the condSg id
function def = sdx_addDep_definition (def,dep, varargin)
def.depList($+1) = dep;

if length(varargin) == 1 then
  csgId = varargin(1); depId = length(def.depList);
  csg = def.condSubgraphList(csgId);
  csg = sdx_addDep_condSubgraph (csg, depId);
  def.condSubgraphList(csgId) = csg;
end
endfunction


//*** general get functions ***//

//*** Sdx_conditionned_definition specific functions ***//

// added at creation, do NOT self add
function def = sdx_setCondPortId_definition (def, portId)
if typeof(def) <> "Sdx_conditionned_definition" then
  errStr = msprintf("cannot add condPort to def %s which of type %s",def.name,typeof(def));
  error (errStr);
end
if typeof (portId) <> "constant" then error ("wrong type for portId"); end

// str = msprintf("redefining cond port for def %s", def.name);
//warning(str);
def.condPortId = portId;
endfunction

function pt = sdx_getCondPort_definition (def)
ptId = sdx_getCondPortId_definition (def);
pt = def.IportList(ptId);
endfunction

function pt = sdx_getCondPortId_definition (def)
if typeof(def) <> "Sdx_conditionned_definition" then error ("cannot get cond port in non conditionned definition"); end
pt = def.condPortId;
endfunction

function def = sdx_addCondSubgraph_definition (def,sg)
if typeof(def) <> "Sdx_conditionned_definition" then error ("cannot add sg in a non conditionned definition"); end
if typeof(sg) <> "Sdx_condSubgraph" then error("incorrect type for sg"); end

def.condSubgraphList($+1) = sg;
endfunction

function def = sdx_setSize_definition (def,x,y)
def.sizeX = x; def.sizeY = y;
endfunction




//*** Finalization functions ***//


// this function MUST be called at the end of apply construction 
// _before_ the toString processing.
// it checks and replace all dependences fields by the corresponding objects (refs and ports)
// when some fields like references or definitions are referenced by their id/name
// in either references or dependences
function mainApp = sdx_process_mainApp (mainApp)
// scan all definitions for ref resolution
// ALGO:

// FOR each definition
for i=1:size(mainApp.definitionList)
  def = mainApp.definitionList(i);
  
  for j=1:size(def.refList)   // ... for each reference
    ref = def.refList(j);  // ... retrieve the definition object
    ref.definition = sdx_getDef_mainApp (mainApp, ref.definition);
    def.refList(j) = ref;
  end
  // could also check for each reference if its def exists in mainApp
  // ...  FOR each dependence ...
  
  for j=1:size(def.depList) 
    dep = def.depList(j);
    // ... get reference object for source
    
    // if the connection is from a local port, the srcPort is an input port
    
    if sdx_refIsLocal (dep.srcBlock) then
      dep.srcBlock = ""; // convention : emptystr() <=> local port
      dep.srcPort = sdx_getIPortName_definition (def,dep.srcPort);
    else
      srcBlk = sdx_getRef_definition (def, dep.srcBlock);
      dep.srcBlock = srcBlk.name;
      dep.srcPort = sdx_getOPortName_definition (srcBlk.definition, dep.srcPort);
    end
    
    if sdx_refIsLocal (dep.destBlock) then
      dep.destBlock = "";
      dep.destPort = sdx_getOPortName_definition (def,dep.destPort);
    else
      destBlk = sdx_getRef_definition (def, dep.destBlock);
      dep.destBlock = destBlk.name;
      dep.destPort = sdx_getIPortName_definition (destBlk.definition, dep.destPort);
    end
    
    //printf("DEBUG: dep: <%s . %s -> %s . %s>\n",..

    //dep.srcBlock,dep.srcPort, dep.destBlock, dep.destPort);
    
    def.depList(j) = dep;
  end
  //  def = sdx_orderPorts_definition (def); // re ordering i and o ports => no more needed
  mainApp.definitionList(i) = def;
end
endfunction

// general conventions about special naming in dependences
// Note that no dependence is given as argument
//function ok = sdx_portIsCond (port)
//ok =%F;
//if typeof (port) == "constant" then
//  if port == 0 then ok = %T; end
//end
//endfunction

function ok = sdx_refIsLocal (ref)
ok = %F;
select typeof(ref),
case "string" then
  ok = (ref == emptystr());
case "Sdx_reference" then
  if ref.name == emptystr() then 
    warning ("sdx_isLocal_reference : deprecated usage");
    ok = %T;
  end
case "constant" then
  if ref <= 0 then 
    warning ("sdx_isLocal_reference : VERY deprecated usage"); 
    ok = %T; 
  end
else
  error ("wrong type for reference");
end
endfunction

function realRef = sdx_getRef_definition (def, ref)
select typeof(ref),
case "constant" then
  realRef = def.refList(ref);
case "string" then
  for currRef=def.refList
    if currRef.name == ref then
      realRef = currRef; return;
    end
  end
  str = msprintf ("ref %s not found in def %s !",ref,def.name);
  error(str);
case "Sdx_reference" then
  realRef = ref;
else error ("wrong ref type");
end
endfunction

function ptName = sdx_getIPortName_definition (def, pt)
select typeof(pt),
case "constant" then // port id
  ptName = def.IportList(pt).name;
case "string" then // port name
  ptName = pt;
case "Sdx_port" then // port object
  ptName = pt.name;
else error("wrong port type");
end
endfunction

function ptName = sdx_getOPortName_definition (def, pt)
select typeof(pt),
case "constant" then // port id
    ptName = def.OportList(pt).name;
case "string" then // port name
  ptName = pt;
case "Sdx_port" then // port object
  ptName = pt.name;
else error("wrong port type");
end
endfunction



// to string 
//


function str = sdx_definition_toString (def)
portStr = emptystr();

// port list to string
for i=1:size(def.IportList)
  pt = def.IportList(i);
  portStr = portStr + "? " + sdx_port_toString(pt, i) + "\n";
end
for i=1:size(def.OportList)
  pt = def.OportList(i);
  portStr = portStr + "! " + sdx_port_toString(pt,i) + "\n";
end

// parameters to string
paramStr = sdx_parameters_toString(def.parameters);

// subgraphs to string
select typeof(def),
case "Sdx_atomic_definition" then // definition is atomic
  // info on cond sg replaced by info on code generation phases. Quite ugly but effective
  if def.algoType == "algorithm" then
    condSgStr = "code_phases:  loopseq initseq endseq;\n";
  else
    condSgStr = emptystr();
  end
case "Sdx_subgraph_definition" then // definition is a non conditionned subgraph
  condSgStr = "conditions: true;\n";
  // references
  condSgStr= condSgStr+ "references:\n";
  for ref=def.refList
    condSgStr = condSgStr + sdx_reference_toString (ref) + "\n";
  end
  // dependences
  condSgStr= condSgStr+ "dependences:\n";
  for dep=def.depList
    condSgStr = condSgStr + sdx_dependence_toString (dep) + "\n";
  end
  
case "Sdx_conditionned_definition" // definition is a conditionned subgraph
  condSgStr = emptystr();
  for sgId=1:length(def.condSubgraphList)
    condSgStr = condSgStr + sdx_condSubgraph_toString(def, sgId);
  end
else
  error("not a valid def type");
end

str = "def " + def.algoType + " " + def.name + " " + paramStr + " : \n"+..
    portStr  + condSgStr;
endfunction

// update
function str = sdx_condSubgraph_toString(def, sgId)
sg = def.condSubgraphList(sgId);

condPt = sdx_getCondPort_definition(def);
condPtName = condPt.name;
condValue = sg.condValue;
// condition
str = "conditions: "+ condPtName + " = " + var_toString(condValue) + ";\n";
// references
str = str + "references:\n";
for id=sg.refPtr
  ref = def.refList(id);
  str = str + sdx_reference_toString (ref) + "\n";
end
// dependences
str = str + "dependences:\n";
for id=sg.depPtr
  dep = def.depList(id);
  str = str + sdx_dependence_toString (dep) + "\n";
end
endfunction
//////////////////////////////////////////////////



////////// conditionned subgraph ////////////
// this struct should be 'private' as it is an inner structure only used in
// conditionned definitions.
// User should first create a tmpCondSubgraph then use the convert function.
/// fields :
// refPtr : pointer to the def's refs
// depPtr : pointer to the def's deps
// condvalue : corresponding cond port value for the csg


function sdxCondSG = sdx_create_condSubgraph (condValue, refPtr, depList)
sdxCondSG = tlist (["Sdx_condSubgraph";"condValue";"refPtr";"depPtr"],..
    condValue, refPtr, depList);
endfunction

function sdxCondSG = sdx_createVoid_condSubgraph ()
sdxCondSG = sdx_create_condSubgraph (-1, list(), list());
endfunction

function sdxCondSG = sdx_fastCreate_condSubgraph (condValue)
sdxCondSG = sdx_create_condSubgraph (condValue, list (), list());
endfunction

function sg = sdx_setCondValue_condSubgraph (sg, condValue)
  sg.condValue = condValue;
endfunction

function sg = sdx_addRef_condSubgraph (sg, refId)
sg.refPtr($+1) = refId;
endfunction

function sg = sdx_addDep_condSubgraph (sg, depId)
sg.depPtr($+1) = depId;
endfunction

// no toString function -> integrated into def toString
//
///////////////////////////////////////////////////////



///////////////// dependence ///////////////////////
function sdxDep = sdx_create_dependence (srcBlk, srcPt, destBlk, destPt)
sdxDep = tlist (["Sdx_dependence";"srcBlock";"srcPort";"destBlock";"destPort"],..
    srcBlk, srcPt, destBlk, destPt);
endfunction

// Here, all the dep fields are assumed to be strings
function str = sdx_dependence_toString (dep)
pointSrc = emptystr(); pointDest = emptystr();
srcBlock = dep.srcBlock; srcPort = dep.srcPort;
destBlock = dep.destBlock; destPort = dep.destPort;

if srcBlock <> emptystr() then pointSrc = "."; end
if destBlock <> emptystr() then pointDest = "."; end

str = msprintf("strong_precedence_data %s%s%s -> %s%s%s;",..
    srcBlock,pointSrc,srcPort,destBlock,pointDest,destPort);
endfunction

function localBlk = sdx_getLocalBlk_dependance ()
localBlk = emptystr();
endfunction
////////////////////////////////////////////////////////


///////////// tmp condSubgraph /////////////////////////
// NEW structure designed to facilitate condSubgraph creation.
// build it like a definition then use the sdx_insertTmpCsg_definition function.
//
// no verifications for now
function tmpCsg = sdx_create_tmpCsg (condValue, refList, depList)
tmpCsg = tlist(["Sdx_tmpCsg";"condValue";"refList";"depList"],..
    condValue, refList, depList);
endfunction
function tmpCsg = sdx_createVoid_tmpCsg ()
tmpCsg = sdx_create_tmpCsg (-1, list(), list());
endfunction
function tmpCsg = sdx_fastCreate_tmpCsg (condValue)
tmpCsg = sdx_create_tmpCsg (condValue, list(), list())
endfunction

function tmpCsg = sdx_setCondValue_tmpCsg (tmpCsg, value)
tmpCsg.condValue = value;
endfunction

function tmpCsg = sdx_addRef_tmpCsg (tmpCsg, ref)
tmpCsg.refList($+1) = ref;
endfunction

function tmpCsg = sdx_addDep_tmpCsg (tmpCsg, dep)
if typeof(dep) <> "Sdx_dependence" then error("wrong type for dep"); end
tmpCsg.depList($+1) = dep;
endfunction

// this function translate informations belonging to a tmpCsg into a definition
// adding the necessary defs, deps and condSg
function def = sdx_insertTmpCsg_definition (def, tmpCsg)

select typeof(def),
case "Sdx_conditionned_definition" then
  // creation of the new subgraph to be added to def
  condSg = sdx_fastCreate_condSubgraph (tmpCsg.condValue);
  // adding refs to the def and adding ptr info into condSg
  for ref=tmpCsg.refList
    // adding ref
    def = sdx_addRef_definition (def,ref);
    currId = length(def.refList);
    // updating csg
    condSg = sdx_addRef_condSubgraph (condSg, currId);
  end
  
  // taking care of dependances
  // note that all references used in tmpCsg.depList are present into def
  for dep=tmpCsg.depList
    // adding dep
    def = sdx_addDep_definition (def,dep);
    currId = length(def.depList);
    // updating csg
    condSg = sdx_addDep_condSubgraph (condSg, currId);
  end
  
  // adding condSg to def
  def = sdx_addCondSubgraph_definition (def,condSg);
  
case "Sdx_subgraph_definition" then
  // adding refs to the def and adding ptr info into condSg
  for ref=tmpCsg.refList
    // adding ref
    def = sdx_addRef_definition (def,ref);
  end
  
  // taking care of dependances
  // note that all references used in tmpCsg.depList are present into def
  for dep=tmpCsg.depList
    // adding dep
    def = sdx_addDep_definition (def,dep);
  end
else
  error("cannot add a cond subgraph in an atomic def");
end
endfunction


////////////////////////////////////////////////////////




/////////////////// MAIN APPLICATION //////////////////////////
function sdxApp = sdx_create_mainApp ()
sdxApp = tlist(["Sdx_mainApplication";"definitionList";"libList";"mainDefId";..
	"extraDurationList"],..
    list(), list(), -1,..
    list()); // archi specific.
endfunction

function sdxApp = sdx_setMainAlgoId_mainApp (sdxApp, mainAlgoId)
sdxApp.mainDefId = mainAlgoId;
endfunction

function id = sdx_getMainAlgoId_mainApp (sdxApp)
id = sdxApp.mainDefId;
endfunction

function sdxApp = sdx_addDefinition_mainApp (sdxApp, def)
sdxApp.definitionList($+1) = def;
endfunction

// function allowing to replace an existing definition by another
function sdxApp = sdx_updateDefinition_mainApp (sdxApp, def, id)
sdxApp.definitionList(id) = def;
endfunction

function id = sdx_getLastDefinitionId_mainApp (sdxApp)
id = length(sdxApp.definitionList);
endfunction

// add a list of definitions at once
function sdxApp = sdx_addDefinitionList_mainApp (sdxApp, defList)
if length(sdxApp.definitionList) > 0 then
  warning("Some definitions are already present!");
end
sdxApp.definitionList = defList;
endfunction

function sdxApp = sdx_addLibrary_mainApp (sdxApp, libName)
sdxApp.libList($+1) = libName;
endfunction

// finalization functions
function sdxApp = sdx_setDefsLibField (sdxApp, libName)
error ("not implemented yet");
endfunction

function def = sdx_getDef_mainApp (mainApp, def)
select typeof(def),
case "constant" then // ref id => retrieving def
  def = mainApp.definitionList(def);
case "string" then // retrieving def O(n)
  for currDef=mainApp.definitionList
    if currDef.name == def then 
      def = currDef; return;
    end
  end
  str = msprintf ("def %s not found !",def);
  error(str);
else
  if sdx_isA_definition(def) then 
    // could check coherency between def provided and actual def
  else
    error ("unknown type");
  end
end 
endfunction




/// main to string function
function str = sdx_mainApp_toString (mainApp)
str = "syndex_version : ""6.6.0""\n"+..
    "application description : ""Algorithm generated by the Scicos To SynDEx translator""\n"+..
    "\n# Generated by Scicos=>SynDEx translator\n\n# Libraries\n";

for sdxLib=mainApp.libList
  str = str + "\ninclude """+sdxLib+""";\n";
end

str = str + "\n\n# Algorithms\n";
for def=mainApp.definitionList
  defStr = sdx_definition_toString(def);
  str = str + "\n" + defStr;
end

mainDefId = sdx_getMainAlgoId_mainApp (sdxApp);
if mainDefId > 0 then
  str = str +..
      "\n# Main Algorithm / Main Architecture\n" +..
      "main algorithm " + sdx_getMainDefName_mainApp(mainApp) + ";\n";
else
  str = str + "\n# NO MAIN ALGO DEFINED !\n";
end

str = str + sdx_architecture_toString(mainApp);
endfunction



function mainDefName = sdx_getMainDefName_mainApp (sdxApp)
mainDefId = sdx_getMainAlgoId_mainApp (sdxApp);
if mainDefId == -1 then
  warning('No main def found !!');
  mainDefName = "";
else
  mainDef = sdx_getDef_mainApp (sdxApp, mainDefId);
  mainDefName = mainDef.name;
end
endfunction

////////////////////////////////////////////////////////////



/////////////////////////////////////////////////////////////////////////
// generic toString functions
/////////////////////////////////////////////////////////////////////////

function coordStr = sdx_coord_toString (x, y, varargin)
coordStr = emptystr();
if size(varargin) == 1 then prefixStr = varargin(1); else prefixStr = emptystr(); end
if x <> emptystr() then // assuming both X and Y are given
  coordStr = prefixStr + var_toString(x) + "," + var_toString(y);
end
endfunction

// should be a 'private' function
function str = sdx_parameter_toString (param)
str = emptystr();
select size(param), // is always a list
case 0 then
  warning("size of param = 0");

  // remarque: ca tombe bien que le cas d'une liste de string de taille 1 corresponde 
  // (pour la surcharge de '()')  un scalaire qui lui, quivaut  un tableau de 
  // taille 1
case 1 then
  str = var_toString(param(1));
else
  str = msprintf("{%s",var_toString(param(1)));
  for i=2:size(param)
    str = msprintf("%s,%s",str,var_toString(param(i)));
  end
  str = msprintf("%s}",str);
end
endfunction


function str = sdx_parameters_toString (params, varargin)
str = emptystr();
select size(params),
case 0 then
  // warning("param list is of size 0");
else
  str = msprintf("<%s",sdx_parameter_toString(params(1)));
  for i=2:size(params)
    str = msprintf("%s;%s",str,sdx_parameter_toString(params(i)));
  end
  str = msprintf("%s>",str);
end
endfunction

function str = var_toString (var)
select typeof(var)
case "constant" then
  str = msprintf("%d",var);
case "string" then
  str = msprintf("%s",var);
else
  warning("var is neither a consant nor a string");
  str = emptystr();
end
endfunction



/////////////////////////////////////////////////////////////////////////
// global functions
// s: a string
/////////////////////////////////////////////////////////////////////////

// MODIF 22/12/05 : refactoring
function ok = isNumber (s)
ok = cmn_isNumber (s)
endfunction

// check if s is the empty string
function ok = isEmpty (s)
ok = cmn_isEmpty (s);
endfunction


// ************** architecture specific functions ************* //

// this toString function is simplified for now. 
// We don't take into account the operators/media 
// types since we only use C code
function str = sdx_architecture_toString (mainApp)
rootOperator = codeGen_getProcName (); // create_codeGen_files.sci
str = "\n# Architectures\n" +..
    "def architecture sciSyn_archi :\n"+..
    "operators:\n"+..
    " u/U "+rootOperator+" ;\n"+..
    " main operator "+rootOperator+";\n"+..
    "medias:\nconnections:\n"+.. // insert here if media needed
    "\n# Main Algorithm / Main Architecture\n"+..
    "main architecture sciSyn_archi;\n";

// add random extra durations in order to enable adequation
str = str + "\n# Extra durations : randomly generated\nextra_durations_operator u/U :\n";
for defId=1:length(mainApp.definitionList)
  def = mainApp.definitionList(defId);
  if sdx_isAtomic_definition (def)
    defName = def.name;
    duration = modulo(defId,3) +1; // random duration
    str = str + msprintf("%s = %d;",defName,duration) + "\n";
  end
end
//condI and condO
str = str + "CondI = 1;\nCondO = 1;\n";

//MODIF 13/01/06
// adding tcp media durations in case of multiprocessor implantation
str = str + "extra_durations_media u/TCP :\nprec_synchro = 2;\ndouble = 2;\n";
endfunction
  
// ****************************************************** //


//////////////// UNIT TESTS AND DEBUG ////////////////////
function ok = toSyndex ()

sdxApp = sdx_create_mainApp ();

sdxApp = sdx_addLibrary_mainApp (sdxApp, "u.sdx")

//// TEST 1 : atomic definitions
//// a/
// def creation
def1 = sdx_createVoid_atomicDefinition ();
// creation/add ports
def1 = sdx_fastAddIport_definition (def1, 1);
def1 = sdx_fastAddIport_definition (def1, 1);
def1 = sdx_fastAddOport_definition (def1, 1);
// other infos
def1 = sdx_setAlgoType_definition (def1, "algorithm");
def1 = sdx_setName_definition (def1,"myDef_1a");

sdxApp = sdx_addDefinition_mainApp (sdxApp, def1);



//// b/
def2 = sdx_createVoid_atomicDefinition ();
// parameter
def2 = sdx_addParameters_definition (def2, list("Sz", "dataList"));
// ports
def2 = sdx_fastAddIport_definition (def2, "Sz");
def2 = sdx_fastAddIport_definition (def2, 1);
def2 = sdx_fastAddOport_definition (def2, "Sz");
// other infos
def2 = sdx_setAlgoType_definition (def2, "algorithm");
def2 = sdx_setName_definition (def2,"myDef_1b");
def2 = sdx_setSize_definition (def2,23,54);

sdxApp = sdx_addDefinition_mainApp (sdxApp, def2);



//// TEST 2 : hierachical definitions
//// a/
def3 = sdx_createVoid_subgraphDefinition ();
def3 = sdx_setName_definition (def3,"myDef_2b");
def3 = sdx_addParameter_definition (def3, "innerSz");
// ports : using ipt and opt
def3 = sdx_fastAddIport_definition (def3, 1);
def3 = sdx_fastAddIport_definition (def3, "innerSz");
def3 = sdx_fastAddOport_definition (def3, "innerSz");
// creation/add refereces
ref1 = sdx_create_reference (def1, "ref1_def1", list());
ref2 = sdx_create_reference (def2, "ref2_def2", list("innerSz",list(4343,3434,'""bla""',5454)));
def3 = sdx_addRef_definition (def3, ref1);
def3 = sdx_addRef_definition (def3, ref2);
// creation/add dependences
dep2 = sdx_create_dependence ("", 1, "ref1_def1",1);
dep0 = sdx_create_dependence ("",2,ref2,1);
dep3 = sdx_create_dependence ("", 1, ref1, 2);
dep1 = sdx_create_dependence (ref1, 1, ref2, 2);
dep4 = sdx_create_dependence (ref2, 1, "", 1);

def3 = sdx_addDep_definition (def3,dep0);
def3 = sdx_addDep_definition (def3,dep1);
def3 = sdx_addDep_definition (def3,dep2);
def3 = sdx_addDep_definition (def3,dep3);
def3 = sdx_addDep_definition (def3,dep4);
// other infos
def3 = sdx_setSize_definition (def3,233,154);

sdxApp = sdx_addDefinition_mainApp (sdxApp, def3);


//// TEST 3 : conditionned hierarchical definition
//// a/
def4 = sdx_createVoid_conditionnedDefinition();
def4 = sdx_addParameter_definition (def4, list("MainSize"));
def4 = sdx_setName_definition (def4, "myDef_3");
//ports
def4 = sdx_fastAddIport_definition (def4, 1);
def4 = sdx_fastAddIport_definition (def4, 1);
def4 = sdx_fastAddIport_definition (def4, "MainSize");

def4 = sdx_fastAddOport_definition (def4, "MainSize");

// 1st tmp subgraph
sg1 = sdx_fastCreate_tmpCsg (1);

ref = sdx_create_reference (def3, "ref2b",list("MainSize"));
sg1 = sdx_addRef_tmpCsg (sg1, ref);

dep = sdx_create_dependence ("",1,ref,1);
dep2 = sdx_create_dependence ("",3,ref,2);
dep3 = sdx_create_dependence (ref,1,"",1);
sg1 = sdx_addDep_tmpCsg (sg1, dep);sg1 = sdx_addDep_tmpCsg (sg1, dep2);sg1 = sdx_addDep_tmpCsg (sg1, dep3);
//insertion 1st tmp subgraph
def4 = sdx_insertTmpCsg_definition (def4, sg1);

// 2nd tmp subgraph
sg2 = sdx_fastCreate_tmpCsg (2);

ref1 = sdx_create_reference (def1,"refToDef1",list());
ref2 = sdx_create_reference (def2,"refToDef2",list("MainSize",list("poipoi","oiuoiu")));
sg2 = sdx_addRef_tmpCsg (sg2,ref1);
sg2 = sdx_addRef_tmpCsg (sg2,ref2);

dep1 = sdx_create_dependence ("",1,ref1,1);
dep2 = sdx_create_dependence ("",2,ref1,2);
dep3 = sdx_create_dependence ("",3,ref2,1);
dep4 = sdx_create_dependence (ref1,1,"refToDef2",2);
dep5 = sdx_create_dependence (ref2,1,"",1);
sg2 = sdx_addDep_tmpCsg (sg2,dep1);
sg2 = sdx_addDep_tmpCsg (sg2,dep2);
sg2 = sdx_addDep_tmpCsg (sg2,dep3);
sg2 = sdx_addDep_tmpCsg (sg2,dep4);
sg2 = sdx_addDep_tmpCsg (sg2,dep5);
// insertion 2nd tmp subgraph
def4 = sdx_insertTmpCsg_definition (def4, sg2);

sdxApp = sdx_addDefinition_mainApp (sdxApp, def4);

sdxApp = sdx_setMainAlgoId_mainApp (sdxApp, length(sdxApp.definitionList));

sdxApp = sdx_process_mainApp (sdxApp);
str = sdx_mainApp_toString (sdxApp);
printf(str);

///////////////////////
pause;

fnam = "/local/cfaure/RESULTATS/temp.sdx";
[fd, err]= mopen(fnam, "w");
if err<> 0 then 
  str = sprintf("Error opening file");
  error (str);
end

mfprintf(fd, str);

mclose(fd);

printf("Finished to write into file %s\n",fnam);
ok = %T;
endfunction
